/*****************************************************************************
 ** ANGRYBIRDS AI AGENT FRAMEWORK
 ** Copyright (c) 2015, XiaoYu (Gary) Ge, Jochen Renz,Stephen Gould,
 **  Sahan Abeyasinghe,Jim Keys, Rohan Verma, Andrew Wang, Peng Zhang
 ** All rights reserved.
**This work is licensed under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
**To view a copy of this license, visit http://www.gnu.org/licenses/
 *****************************************************************************/
package ab.demo;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

import ab.demo.other.ClientActionRobot;
import ab.demo.other.ClientActionRobotJava;
import ab.planner.TrajectoryPlanner;
import ab.vision.ABObject;
import ab.vision.ABType;
import ab.vision.GameStateExtractor.GameState;
import ab.vision.Vision;
//Naive agent (server/client version)

public class ClientNaiveAgent implements Runnable {
	public enum Competition_State {Connecting, Bidding, Running, PostShot, Loading, Waiting, Finished};


	//Wrapper of the communicating messages
	private ClientActionRobotJava ar;
	public byte currentLevel = -1;
	public int failedCounter = 0;
	public int[] solved;
	TrajectoryPlanner tp; 
	private int id = 28888;
	private boolean firstShot;
	private int[] bids;
	private int[] levels;
	private int current_level;
	private int opponent;
	private int round = 1;
	private Competition_State stage;
	private Point prevTarget;
	private Random randomGenerator;
	/**
	 * Constructor using the default IP
	 * */
	public ClientNaiveAgent() {
		// the default ip is the localhost
		ar = new ClientActionRobotJava("127.0.0.1");
		tp = new TrajectoryPlanner();
		randomGenerator = new Random();
		prevTarget = null;
		firstShot = true;
		stage = Competition_State.Loading;

	}
	/**
	 * Constructor with a specified IP
	 * */
	public ClientNaiveAgent(String ip) {
		ar = new ClientActionRobotJava(ip);
		tp = new TrajectoryPlanner();
		randomGenerator = new Random();
		prevTarget = null;
		firstShot = true;
		stage = Competition_State.Loading;

	}
	public ClientNaiveAgent(String ip, int id)
	{
		ar = new ClientActionRobotJava(ip);
		tp = new TrajectoryPlanner();
		randomGenerator = new Random();
		prevTarget = null;
		firstShot = true;
		stage = Competition_State.Loading;
		this.id = id;
	}
	public int getNextLevel()
	{
		int level = 0;
		boolean unsolved = false;
		//all the level have been solved, then get the first unsolved level
		for (int i = 0; i < solved.length; i++)
		{
			if(solved[i] == 0 )
			{
					unsolved = true;
					level = i + 1;
					if(level <= currentLevel && currentLevel < solved.length)
						continue;
					else
						return level;
			}
		}
		if(unsolved)
			return level;
	    level = (currentLevel + 1)%solved.length;
		if(level == 0)
			level = solved.length;
		return level; 
	}
    /* 
     * Run the Client (Naive Agent)
     */
	private void checkMyRoundScore()
	{
		
		int[] scores = ar.getMyRoundScore();
		int[] opponents_scores = ar.getOpponentScore();
		System.out.println(" My score: ");
		for (int level = 0; level < levels.length; level++) {
			boolean won = (Math.abs(bids[level]) == opponents_scores[level] || opponents_scores[level] == 0);
			System.out.println((won ? "WON" : "LOST") 
					+  " level " + levels[level] 
					+ " my score: " + scores[level] 
					+ " - my opponents score: " + opponents_scores[level]);
		}
	}
	
	private void checkScoreBoard() {
		HashMap<Integer,HashMap<Integer,Integer[]>> scoreBoard = ar.getScoreBoard();
		
		System.out.println("==SCOREBOARD==");
		for (int i = 0; i < scoreBoard.keySet().size(); i++) {
			System.out.println("Round " + (i));
			for (Integer key : scoreBoard.get(i).keySet()) {
				System.out.print("Agent " + key + ": ");
				for (int j = 0; j < levels.length; j++) {
					System.out.print("Level " + levels[j] + "(" + scoreBoard.get(i).get(key)[j] + ") ");
				}
				System.out.println();
			}
		}
		System.out.println("==============");
	}
	
	
	private Competition_State getCompetitionState() {
		byte[] bytes = ar.getCompetitionState();
		return Competition_State.values()[ByteBuffer.wrap(bytes).getInt()];
	}
	
	private boolean getTurn() {
		return (ByteBuffer.wrap(ar.getTurn()).getInt() == 1);
	}
	
	private void change_stage(Competition_State new_stage) {
		if (stage != new_stage) {
			switch(stage) {
			case Connecting:
				levels = ar.getLevels();
				bids = new int[levels.length];
				opponent = ar.getOpponent();
				System.out.println("my opponent is: " + opponent);
				System.out.println("we will play on levels: " + levels[0]+", "+ levels[1]+", "+ levels[2]);
				current_level = 0;
				break;
			case Bidding:
				if (new_stage == Competition_State.Running) {
					firstShot = false;
					if (bids[current_level] >= 0 && getTurn()) {
						System.out.println("I won the bid to go first");
						firstShot = true;
					}
					if (bids[current_level] < 0 && !getTurn()) {
						System.out.println("I won the bid to go second");
					}
				}
				break;
			case PostShot: case Running:
				if (new_stage == Competition_State.Loading || new_stage == Competition_State.Bidding) {
					System.out.println("Level" + levels[current_level] + " completed");
					current_level = current_level + 1;
					checkMyRoundScore();
				}
				if (new_stage == Competition_State.Finished) {
					System.out.println("I finished playing again " + opponent);
				}
				break;
			case Finished:
				System.out.println("Round " + round + "finished");
				round = round + 1;
				checkScoreBoard();
				opponent = ar.getOpponent();
				current_level = 0;
				bids = new int[levels.length];
				
				break;
			default:
				if (new_stage == Competition_State.Finished) {
					System.out.println("I finished playing again " + opponent);
				}
				break;
			}
			stage = new_stage;
			System.out.println(stage.toString());
		}
	}
	
	public void run() {	
		byte[] info = ar.configure(ClientActionRobot.intToByteArray(id));
		
		
		while (true) {
			change_stage(getCompetitionState());
			try{
				switch (stage) {
				case Bidding:
					if (getTurn()) {
						Integer value = estimate();
						
						System.out.println("estimated: " +value);
						byte[] bid_ok =  ar.bid(ClientActionRobot.intToByteArray(value/2));
						int bid_ok_resp = ByteBuffer.wrap(bid_ok).getInt();
						
						if (bid_ok_resp == 1) {
							bids[current_level] = value;
							System.out.println("made bid");
						}
					} else {
						// Wait for turn or next stage
						try {
							Thread.sleep(500);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
					break;
				case Running:
					if (getTurn()) {
											
							solve();

					} else {
						// Wait for turn or level to finish
						try {
							Thread.sleep(500);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
					break;
				default:
					// Connecting, PostShot, Waiting, Loading, Finished
					if (stage == Competition_State.Finished) {
						//checkScoreBoard();
					}
					try {
						Thread.sleep(500);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					break;
				}
			} 
			catch(InvalidImageException e)
			{
				System.out.print("I am in the wrong state, wait for 2 seconds");
				try {
					Thread.sleep(2000);
				} catch (InterruptedException e1) {
					e.printStackTrace();
				}
			}
			
		}

	}
	
	public Integer estimate() throws InvalidImageException {
		Integer estimate = 0;
		
		// capture Image
		BufferedImage screenshot = ar.doScreenShot();
		
		// process image
		Vision vision = new Vision(screenshot);
		
		Rectangle sling = vision.findSlingshotMBR();


		//If the level is loaded (in PLAYING　state)but no slingshot detected, then the agent will request to fully zoom out.
		while (sling == null && ar.checkState() == GameState.PLAYING) {
			if (getCompetitionState() != Competition_State.Bidding || !getTurn()) {
				return 0;
			}
				
			System.out.println("no slingshot detected. Please remove pop up or zoom out");
			
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				
				e.printStackTrace();
			}
			ar.fullyZoomOut();
			screenshot = ar.doScreenShot();
			vision = new Vision(screenshot);
			sling = vision.findSlingshotMBR();
		}
		
		 // get all the pigs
		List<ABObject> pigs = vision.findPigsMBR();
		
		// get all birds
		List<ABObject> birds = vision.findBirdsMBR();
		
		estimate = 10000*(birds.size() - pigs.size()) + 5000*pigs.size();
		
		return estimate;
	}


	  /** 
	   * Solve a particular level by shooting birds directly to pigs
	   * @return GameState: the game state after shots.
     */
	public void solve() throws InvalidImageException

	{
		// capture Image
		BufferedImage screenshot = ar.doScreenShot();
		
		
		if (screenshot == null) {
			return;
		}

		// process image
		Vision vision = new Vision(screenshot);
		
		Rectangle sling = vision.findSlingshotMBR();

		//If the level is loaded (in PLAYING　state)but no slingshot detected, then the agent will request to fully zoom out.
		while (sling == null && ar.checkState() == GameState.PLAYING) {
			if (getCompetitionState() != Competition_State.Running || !getTurn()) {
				System.out.println("unable to locate sling and make shot in time, server will make a naive shot");
				return;
			}
			
			System.out.println("no slingshot detected. Please remove pop up or zoom out");
			
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				
				e.printStackTrace();
			}
			ar.fullyZoomOut();
			screenshot = ar.doScreenShot();
			if (screenshot == null) {
				return;
			}
			vision = new Vision(screenshot);
			sling = vision.findSlingshotMBR();
		}

		System.out.println("found sling");
		
		 // get all the pigs
 		List<ABObject> pigs = vision.findPigsMBR();
 		
		// if there is a sling, then play, otherwise skip.
		if (sling != null) {		
			//If there are pigs, we pick up a pig randomly and shoot it. 
			if (!pigs.isEmpty()) {		
				Point releasePoint = null;
				// random pick up a pig
					ABObject pig = pigs.get(randomGenerator.nextInt(pigs.size()));
					
					Point _tpt = pig.getCenter();

					
					// if the target is very close to before, randomly choose a
					// point near it
					if (prevTarget != null && distance(prevTarget, _tpt) < 10) {
						double _angle = randomGenerator.nextDouble() * Math.PI * 2;
						_tpt.x = _tpt.x + (int) (Math.cos(_angle) * 10);
						_tpt.y = _tpt.y + (int) (Math.sin(_angle) * 10);
						System.out.println("Randomly changing to " + _tpt);
					}

					prevTarget = new Point(_tpt.x, _tpt.y);

					// estimate the trajectory
					ArrayList<Point> pts = tp.estimateLaunchPoint(sling, _tpt);

					// do a high shot when entering a level to find an accurate velocity
					if (firstShot && pts.size() > 1) {
						releasePoint = pts.get(1);
					} else 
						if (pts.size() == 1)
							releasePoint = pts.get(0);
						else 
							if(pts.size() == 2)
							{
								// System.out.println("first shot " + firstShot);
								// randomly choose between the trajectories, with a 1 in
								// 6 chance of choosing the high one
								if (randomGenerator.nextInt(6) == 0)
									releasePoint = pts.get(1);
								else
								releasePoint = pts.get(0);
							}
							Point refPoint = tp.getReferencePoint(sling);

					// Get the release point from the trajectory prediction module
					int tapTime = 0;
					if (releasePoint != null) {
						double releaseAngle = tp.getReleaseAngle(sling,
								releasePoint);
						System.out.println("Release Point: " + releasePoint);
						System.out.println("Release Angle: "
								+ Math.toDegrees(releaseAngle));
						int tapInterval = 0;
						switch (ar.getBirdTypeOnSling()) 
						{

							case RedBird:
								tapInterval = 0; break;               // start of trajectory
							case YellowBird:
								tapInterval = 65 + randomGenerator.nextInt(25);break; // 65-90% of the way
							case WhiteBird:
								tapInterval =  50 + randomGenerator.nextInt(20);break; // 50-70% of the way
							case BlackBird:
								tapInterval =  0;break; // 70-90% of the way
							case BlueBird:
								tapInterval =  65 + randomGenerator.nextInt(20);break; // 65-85% of the way
							default:
								tapInterval =  60;
						}
						
						tapTime = tp.getTapTime(sling, releasePoint, _tpt, tapInterval);
						
					} else
						{
							System.err.println("No Release Point Found");
							return;
						}
				
				
					// check whether the slingshot is changed. the change of the slingshot indicates a change in the scale.
					ar.fullyZoomOut();
					screenshot = ar.doScreenShot();
					vision = new Vision(screenshot);
					Rectangle _sling = vision.findSlingshotMBR();
					if(_sling != null)
					{
						double scale_diff = Math.pow((sling.width - _sling.width),2) +  Math.pow((sling.height - _sling.height),2);
						if(scale_diff < 25)
						{
							int dx = (int) releasePoint.getX() - refPoint.x;
							int dy = (int) releasePoint.getY() - refPoint.y;
							if(dx < 0)
							{
								long timer = System.currentTimeMillis();
								ar.shoot(refPoint.x, refPoint.y, dx, dy, 0, tapTime, false);
								firstShot = false;
								System.out.println("It takes " + (System.currentTimeMillis() - timer) + " ms to take a shot");
							}
						}
						else
							System.out.println("Scale is changed, can not execute the shot, will re-segement the image");
					}
					else
						System.out.println("no sling detected, can not execute the shot, will re-segement the image");
				
			}
		}
	}

	private double distance(Point p1, Point p2) {
		return Math.sqrt((double) ((p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y)* (p1.y - p2.y)));
	}

	public static void main(String args[]) {

		ClientNaiveAgent na;
		if(args.length > 0)
			na = new ClientNaiveAgent(args[0]);
		else
			na = new ClientNaiveAgent();
		na.run();
		
	}
}
