/*****************************************************************************
 ** ANGRYBIRDS AI AGENT FRAMEWORK
 ** Copyright (c) 2015, XiaoYu (Gary) Ge, Jochen Renz,Stephen Gould,
 **  Sahan Abeyasinghe,Jim Keys, Rohan Verma, Andrew Wang, Peng Zhang
 ** All rights reserved.
**This work is licensed under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
**To view a copy of this license, visit http://www.gnu.org/licenses/
 *****************************************************************************/

package client.messages;

public class ClientMessageDecoder {
	public ClientMessageTable getMessage(int code) {
		switch (code) {
		case 1:
			return ClientMessageTable.configure;
		case 11:
			return ClientMessageTable.doScreenShot;
		case 51:
			return ClientMessageTable.loadLevel;
		case 52:
			return ClientMessageTable.restartLevel;
		case 31:
			return ClientMessageTable.cshoot;
		case 32:
			return ClientMessageTable.pshoot;
		case 12:
			return ClientMessageTable.getState;
		case 34:
			return ClientMessageTable.fullyZoomOut;
		case 14:
			return ClientMessageTable.getCurrentLevel;
		case 13:
			return ClientMessageTable.getBestScores;
		case 33:
			return ClientMessageTable.shootSeq;
		case 41:
			return ClientMessageTable.cFastshoot;
		case 42:
			return ClientMessageTable.pFastshoot;
		case 43:
			return ClientMessageTable.shootSeqFast;
		case 23:
			return ClientMessageTable.getMyScore;
		case 35:
			return ClientMessageTable.fullyZoomIn;
		case 36:
			return ClientMessageTable.clickInCentre;
		case 53:
			return ClientMessageTable.bid;
		case 54:
			return ClientMessageTable.getOpponent;
		case 55:
			return ClientMessageTable.getCompetitionState;
		case 56:
			return ClientMessageTable.getTurn;
   		case 57:
   			return ClientMessageTable.getOpponentScore;
   		case 58:
   			return ClientMessageTable.getScoreBoard;
   		case 59:
   			return ClientMessageTable.getLevels;
   		case 60:
   			return ClientMessageTable.wasteBird;
   		case 61:
   			return ClientMessageTable.getMyRoundScore;
		}

		return null;
	}

	public int[] parseShoot(byte[] shootMessage) {
		int[] params = new int[6];
		assert (shootMessage.length > 23);
		for (int i = 0; i < 6; i++) {
			params[i] = bytesToInt(shootMessage[0 + i * 4],
					shootMessage[1 + i * 4], shootMessage[2 + i * 4],
					shootMessage[3 + i * 4]);

		}

		return params;

	}

	public int parseId(byte[] id) {
		assert (id.length > 3);
		return bytesToInt(id);
	}

	public int parseBid(byte[] amount) {
		assert (amount.length > 3);
		return bytesToInt(amount);
	}
	
	public String printMessage(byte[] message) {
		int code = message[0];
		String result = "";
		byte[] messageBody;

		switch (code) {
		case 1: {
			result += ClientMessageTable.configure;
			if (message.length > 4) {
				messageBody = new byte[message.length - 1];
				System.arraycopy(message, 1, messageBody, 0, message.length - 1);
				result += "  " + parseId(messageBody);
			}
			return result;
		}
		case 11: {
			return result += ClientMessageTable.doScreenShot;
		}
		case 51: {
			result += ClientMessageTable.loadLevel;
			if (message.length > 1) {
				result += "  " + message[1];
			}
			return result;
		}
		case 52: {
			return result += ClientMessageTable.restartLevel;
		}
		case 53: {
			result += ClientMessageTable.bid;
			if (message.length > 1) {
				result += "  " + message[1];
			}
			return result;
		}
		case 31: {
			result += ClientMessageTable.cshoot;
			if (message.length > 24) {
				messageBody = new byte[message.length - 1];
				System.arraycopy(message, 1, messageBody, 0, message.length - 1);
				for (int param : parseShoot(messageBody))
					result += "  " + param;
			}
			return result;

		}
		case 32: {
			result += ClientMessageTable.pshoot;
			if (message.length > 24) {
				messageBody = new byte[message.length - 1];
				System.arraycopy(message, 1, messageBody, 0, message.length - 1);
				for (int param : parseShoot(messageBody))
					result += "  " + param;
			}
			return result;

		}
		case 41: {
			result += ClientMessageTable.cFastshoot;
			if (message.length > 24) {
				messageBody = new byte[message.length - 1];
				System.arraycopy(message, 1, messageBody, 0, message.length - 1);
				for (int param : parseShoot(messageBody))
					result += "  " + param;
			}
			return result;

		}
		case 42: {
			result += ClientMessageTable.pFastshoot;
			if (message.length > 24) {
				messageBody = new byte[message.length - 1];
				System.arraycopy(message, 1, messageBody, 0, message.length - 1);
				for (int param : parseShoot(messageBody))
					result += "  " + param;
			}
			return result;

		}
		case 43: {
			result += ClientMessageTable.shootSeqFast + "\n";

			if (message.length > 1) {
				int numOfShots = message[1];
				result += " total " + numOfShots + " shots\n";
				byte[] shot = new byte[24];
				for (int i = 0; i < numOfShots; i++) {
					ClientMessageTable shotType = getMessage(message[2 + i * 25]);
					if (shotType != null) {
						System.arraycopy(message, i * 25 + 3, shot, 0,
								shot.length);
						result += shotType + "";
						for (int param : parseShoot(shot))
							result += "  " + param;
						result += "\n";
					} else
						break;
				}

			}
			result += "shootSeq End";
			return result;

		}
		case 12: {
			return result += ClientMessageTable.getState;
		}
		case 34:
			return result += ClientMessageTable.fullyZoomOut;
		case 14:
			return result += ClientMessageTable.getCurrentLevel;
		case 13:
			return result += ClientMessageTable.getBestScores;
		case 23:
			return result += ClientMessageTable.getMyScore;
		case 33: {
			result += ClientMessageTable.shootSeq + "\n";

			if (message.length > 1) {
				int numOfShots = message[1];
				result += " total " + numOfShots + " shots\n";
				byte[] shot = new byte[24];
				for (int i = 0; i < numOfShots; i++) {
					ClientMessageTable shotType = getMessage(message[2 + i * 25]);
					if (shotType != null) {
						System.arraycopy(message, i * 25 + 3, shot, 0,
								shot.length);
						result += shotType + "";
						for (int param : parseShoot(shot))
							result += "  " + param;
						result += "\n";
					} else
						break;
				}

			}
			result += "shootSeq End";
			return result;

		}
		case 35:
			return result += ClientMessageTable.fullyZoomIn;
		case 36:
			return result += ClientMessageTable.clickInCentre;
		}
		return result;

	}

	private int bytesToInt(byte... b) {
		int value = 0;
		for (int i = 0; i < 4; i++) {
			int shift = (4 - 1 - i) * 8;
			value += (b[i] & 0x000000FF) << shift;
		}
		return value;
	}

}
