/*****************************************************************************
 ** ANGRYBIRDS AI AGENT FRAMEWORK
 ** Copyright (c) 2015, XiaoYu (Gary) Ge, Jochen Renz,Stephen Gould,
 **  Sahan Abeyasinghe,Jim Keys, Rohan Verma, Andrew Wang, Peng Zhang
 ** All rights reserved.
**This work is licensed under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
**To view a copy of this license, visit http://www.gnu.org/licenses/
 *****************************************************************************/

package io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import server.ABServer;
import client.logger.Configuration;
import client.logger.PlayerInfo;

public class ScoreIO {
	
	private static void writeScoreToFile(File file, int[] scoreArr)
	{
		try 
		{
				BufferedWriter bw = new BufferedWriter(new FileWriter(file));
				StringBuilder sb = new StringBuilder();
				int level = 0;
				for (int score : scoreArr)
				{
					//Log.echo("save score: " + score);
					sb.append("Level: " + ++level + " Score: " + score + "\n");
				}
				bw.write(sb.toString());
				bw.close();
		}
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	public static void saveTeamScore(File file, Map<Integer, Integer> teamScore)
	{
		int[] scoreArr = new int[ABServer.maxLevelNum];
		for (int i = 0; i < scoreArr.length; i++)
			scoreArr[i] = teamScore.get(i + 1);
		writeScoreToFile(file, scoreArr);
	}
	public static void saveGlobalScore(File file)
	{
		int[] scoreArr = new int[ABServer.maxLevelNum];
		if(ABServer.round > 2) //global score is dynamically updated 
			for (int i = 0; i < ABServer.level; i++)
			{   
				//System.out.println("global score: " + ABServer.global.get(i + 1));
				scoreArr[i] = ABServer.global.get(i + 1);
			}
		else
		//calculate dynamically
		{
			PlayerInfo pinfo = ABServer.player_info;
			for (Configuration conf : pinfo.getPlayers_Info().values())
			{
				for (int level : conf.getScore().keySet())
				{
					HashMap<Integer, Integer> score = conf.getScore();
					if (score.get(level) > scoreArr[level - 1])
					{
						scoreArr[level - 1] = score.get(level);
					}
				}
			}
		}
		
		writeScoreToFile(file, scoreArr);
		
		
	}
	public static int[] loadScore(File file)
	{
		int[] scoreArr = new int[21];
		
		if(file.exists()){
			try {
				System.out.println("Load Score File " + file.getName());
				BufferedReader br = new BufferedReader(new FileReader(file));	
			
				while(br.ready())
				{
					String line = br.readLine();
					String level_str = line.substring(line.indexOf(":") + 2, line.indexOf("S") - 1);
					
					int level = Integer.parseInt(level_str);
				
					String score_str = line.substring(line.lastIndexOf(":") + 2);
					int score = Integer.parseInt(score_str);
					scoreArr[level - 1] = score;
				}
				br.close();
			} catch (FileNotFoundException e) {
				
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
		}
		else
			System.err.println(" Score file " + file.getName() + " cannot be found, will use empty list instead ");
		return scoreArr;
	}
 	public static int[] loadScore(String dir)
	{
		File file = new File(dir);
		return loadScore(file);
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
