/*****************************************************************************
 ** ANGRYBIRDS AI AGENT FRAMEWORK
 ** Copyright (c) 2015, XiaoYu (Gary) Ge, Jochen Renz,Stephen Gould,
 **  Sahan Abeyasinghe,Jim Keys, Rohan Verma, Andrew Wang, Peng Zhang
 ** All rights reserved.
**This work is licensed under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
**To view a copy of this license, visit http://www.gnu.org/licenses/
 *****************************************************************************/

package server;

import java.util.HashMap;

import server.ABServerManager.Competition_State;
import vision.GameStateExtractor.GameState;

public class Game implements Runnable {
	private static final int BIDDING_TIME = 30 * 1000;
	private static final int TURN_TIME = 30 * 1000;
	private static final int POSTSHOT_TIME = 15 * 1000;
	
	AgentThread first;
	int first_id;
	AgentThread second;
	int second_id;
	
	public volatile boolean term = false;
	
	boolean first_turn;
	boolean first_won;
	boolean first_won_bid, second_won_bid;
	
	Competition_State state;
	
	long time;
	
	int current_level;
	
	private HashMap<Integer, Integer[]> scores;
	



	public Game(AgentThread agentThread, int agent_id, AgentThread agentThread2, int agent_id2) {
		first = agentThread;
		first_id = agent_id;
		second = agentThread2;
		second_id = agent_id2;

		first_turn = true;
		state = Competition_State.Connecting;
		current_level = 0;
		scores = new HashMap<Integer, Integer[]>();
		
		// initialise scores
		Integer[] scoreSet1 = {0, 0, 0};
		scores.put(first_id, scoreSet1);
		Integer[] scoreSet2 = {0, 0, 0};
		scores.put(second_id, scoreSet2);
	}

	public void loadNextLevel() {		
		int next_level = ABServerManager.getLevel(current_level);
		if (next_level == -1) {
			state = Competition_State.Finished;
		} else {
			System.out.println(ABServerManager.getLevel(current_level));
			first.loadLevel(ABServerManager.getLevel(current_level));
			state = Competition_State.Loading;
			current_level = current_level + 1;
		}

	}


	public void startBidding() {
		time = System.currentTimeMillis();
		
		first_turn = true;
		first.setTurn(first_turn);
		second.setTurn(!first_turn);
		
		first.new_bid();
		second.new_bid();
		state = Competition_State.Bidding;
	}
	
	public void shotTaken() {
		state = Competition_State.PostShot;
		time = System.currentTimeMillis();
	}

	@Override
	public void run() {
		loadNextLevel();
		while (!term) {
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) 
			{
				e.printStackTrace();
			}
			switch (state) {
				case Finished:
					// Wait
				break;
				case Connecting:
					// Wait
					break;
				case Waiting:
					// Wait
					break;
				case Loading:
					if (first.getGameState() == GameState.PLAYING) {
						startBidding();
					}
					break;
				case PostShot:
					if (first.getGameState() == GameState.WON) {
						first_won = first_turn;
						updateScoreBoard();
						first.GoToLevelSelectionPage();
						loadNextLevel();
					} else if (System.currentTimeMillis() > time + POSTSHOT_TIME) {
						
						first_turn = !first_turn;
						first.setTurn(first_turn);
						second.setTurn(!first_turn);
						
						time = System.currentTimeMillis();
						state = Competition_State.Running;
					}
					break;
				case Bidding:
					if (System.currentTimeMillis() > time + BIDDING_TIME || activeAgent().has_bid()) {
						if (first_turn) {
							
							first_turn = false;
							first.setTurn(first_turn);
							second.setTurn(!first_turn);
							
							time = System.currentTimeMillis();
						} else {
							state = Competition_State.Running;
							
							initialiseTurns();
							time = System.currentTimeMillis();
							
						}
					} 
					break;
				case Running:
					if (System.currentTimeMillis() > time + TURN_TIME) {
						System.out.println("shot time expired wasting bird");
						first.setTurn(false);
						second.setTurn(false);
						if (first_turn) {
							first.wasteBird();
						} else {
							second.wasteBird();
						}
						
						// TODO Make naive shot
						state = Competition_State.PostShot;
						time = System.currentTimeMillis();
					}
					break;
			}
		}
	}
	
	public void shot_made() {
		state = Competition_State.PostShot;
		time = System.currentTimeMillis();
	}

	private AgentThread activeAgent() {
		return (first_turn ? first : second);
	}
	
	private void initialiseTurns() {
		 //System.out.println(first_id + " Goes First. " + second_id + " Goes Second");
		 System.out.println("Bids are " + first_id + ": " + first.getBid() + " and " + second_id + ": " + second.getBid()); 
		  if (first.getBid() > second.getBid()) {
			  first_won_bid = first.getBid() >= 0;
			  second_won_bid = second.getBid() < 0;
			  
			  first_turn = true;
			  first.setTurn(true);
			  second.setTurn(false); 
		  } else if (second.getBid() > first.getBid()) {
			  first_won_bid = first.getBid() < 0;
			  second_won_bid = second.getBid() >= 0;
			  
			  first_turn = false;
			  second.setTurn(true);
			  first.setTurn(false);
		  } else {
			  if (Math.random() > 0.5) {
				  first_won_bid = true;
				  second_won_bid = false;
				  
				  first_turn = true;
				  first.setTurn(true);
				  second.setTurn(false); 
			  } else {
				  first_won_bid = false;
				  second_won_bid = true;
				  
				  first_turn = false;
				  second.setTurn(true);
				  first.setTurn(false);
			  }
		  }
		
	}

	private void updateScoreBoard() {
		  boolean updated = false;
		  while (!updated) {
			  int score = first.checkCurrentScoreSafemode();
			  if (score == -1) {
				  updated = false;
			  } else {
				  int score_first = -1;
				  int score_second = -1;
				  int bid_first = first.getBid();
				  int bid_second = second.getBid();
				  if (first_won) {
					  if (first_won_bid) {
						  score_first = score - Math.abs(bid_first);
						  score_second = Math.abs(bid_first);
					  } else {
						  score_first = score;
						  score_second = 0;
					  }
				  } else {
					  if (second_won_bid) {
						  score_second = score - Math.abs(bid_second);
						  score_first = Math.abs(bid_second);
					  } else {
						  score_second = score;
						  score_first = 0;
					  }
				  }
				  
				  Integer[] scoreSet1 = scores.get(first_id);
				  scoreSet1[current_level-1] = score_first; 
				  scores.put(first_id, scoreSet1);
				  
				  Integer[] scoreSet2 = scores.get(second_id);
				  scoreSet2[current_level-1] = score_second; 
				  scores.put(second_id, scoreSet2);
				  updated = true;
			  }
		  }
	}

	public Competition_State getCompetitionState() {
		return state;
	}

	public HashMap<Integer, Integer[]> getScoreBoard() {
		return scores;
	}

	public int getOpponent(Agent agent) {
		if (first.same_agent(agent)) {
			return second_id;
		} else {
			return first_id;
		}
	}
	public int getMe(Agent agent) {
		if (first.same_agent(agent)) {
			return first_id;
		} else {
			return second_id;
		}
	}
}
