/*****************************************************************************
 ** ANGRYBIRDS AI AGENT FRAMEWORK
 ** Copyright (c) 2015, XiaoYu (Gary) Ge, Jochen Renz,Stephen Gould,
 **  Sahan Abeyasinghe,Jim Keys, Rohan Verma, Andrew Wang, Peng Zhang
 ** All rights reserved.
**This work is licensed under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
**To view a copy of this license, visit http://www.gnu.org/licenses/
 *****************************************************************************/

package server.schema;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Observable;

import javax.imageio.ImageIO;

import org.java_websocket.WebSocket;

import proxy.Proxy;
import proxy.message.ProxyClickMessage;
import proxy.message.ProxyMouseWheelMessage;
import proxy.message.ProxyScreenshotMessage;
import server.ABServer;
import server.Agent;
import server.util.Log;
import server.util.StateUtil;
import vision.GameStateExtractor.GameState;
import vision.Vision;
import client.logger.Configuration;

public class LoadingLevelSchema extends Observable {
	private Proxy server;
	private Configuration conf;
	private StateUtil stateUtil;
	private WebSocket chrome_window;
	private Agent agent;
	private static List<Point> LEVEL_VERFICATION = new ArrayList<Point>(); //No need to synchronize
	private final static int ERROR_NOT_FULLY_ZOOM = -1;
	private final static int ERROR_MISMATCHED_LEVEL = -2;
	private final static int PASS = 0;
    public static void setLevel_Verification(List<Point> level_verification)
    {
    	LEVEL_VERFICATION = level_verification;
    }

    public LoadingLevelSchema(){}
	public LoadingLevelSchema(Agent agent) {
		this.server = agent.getAgentProxy();
		this.conf = agent.getCurrentConfiguration();
		this.chrome_window = agent.getChrome_window();
		this.agent = agent;
		stateUtil = agent.getStateUtil();
	}

	public boolean loadLevel(int i) {
		boolean loaded = true;
		if (i > ABServer.level) 
		{
			i = ((i % ABServer.level) == 0) ? ABServer.level : i % ABServer.level ;

		}
		//if i = 0, and it will be current level
	

		GameState state = stateUtil.checkCurrentState();
		loaded = loadLevel(state, i);

		state = stateUtil.checkCurrentState();
		while (state != GameState.PLAYING) 
		{
			Log.echo("Client " + agent.playerId + " In state: " + state + " Try reloading...");
			if(state == GameState.WON)
			{
				agent.tryRecordScore();
				
			}
			loaded = loadLevel(state, i);
			
			try {
				Thread.sleep(4000);
			} catch (InterruptedException e1) {
				
				e1.printStackTrace();
			}

			state = stateUtil.checkCurrentState();

		}
		return loaded;
	}

	private boolean loadLevel(GameState state, int level) {
		//System.out.println(" start to load level " + level + " State: " + state);
		Log.echo("Client: " + agent.playerId + " Load level: " + level);
		int current_level = conf.getCurrent_level();
		if (level == 0)
			level = current_level;
		else if (level < 1) {
			System.out.println(" Unexpected Level < 1 ");
			return false;
		}
		stateUtil.GoFromMainMenuToLevelSelection();
		//System.out.println("LoadingLevelSchema: loadLevel starts");
		if (state == GameState.WON || state == GameState.LOST) {
			if (state == GameState.WON && level == current_level + 1)
				server.send(chrome_window, new ProxyClickMessage(500, 375)); // go
																				// to
																				// the
																				// next
																				// level
			else {
				server.send(chrome_window, new ProxyClickMessage(342, 382));// Click
																			// the
																			// left
																			// most
																			// button
																			// at
																			// the
																			// end
																			// page
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}

				server.send(chrome_window, new ProxyClickMessage(
						54 + ((level - 1) % 7) * 86,
						110 + ((level - 1) / 7) * 100));
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			/*if (level == 1)
				// skip the animation, the animation does not appear in the SD
				// mode.
				server.send(chrome_window, new ProxyClickMessage(1176, 704));*/
		} else if (state == GameState.PLAYING) {
			server.send(chrome_window, new ProxyClickMessage(48, 44));// Click
																		// the
																		// left
																		// most
																		// button,
																		// pause
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e1) {

				e1.printStackTrace();
			}
			server.send(chrome_window, new ProxyClickMessage(168, 28));// Click
																		// the
																		// left
																		// most
																		// button,
																		// pause
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
			server.send(chrome_window, new ProxyClickMessage(
					54 + ((level - 1) % 7) * 86, 110 + ((level - 1) / 7) * 100));
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {

				e.printStackTrace();
			}
			/*if (level == 1)
				server.send(chrome_window, new ProxyClickMessage(1176, 704));*/
		} else {
			server.send(chrome_window, new ProxyClickMessage(
					54 + ((level - 1) % 7) * 86, 110 + ((level - 1) / 7) * 100));
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			/*if (level == 1)
				server.send(chrome_window, new ProxyClickMessage(1176, 704));*/
		}

		// Wait 9000 seconds for loading the level
		state = stateUtil.checkCurrentState();
		int count = 0; // at most wait 10 seconds
		while (state != GameState.PLAYING && count < 3) {
			try {
				Thread.sleep(3000);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
			count++;
			state = stateUtil.checkCurrentState();

		}

		if (state == GameState.PLAYING) 
		{

		
			conf.setCurrent_level(level);
		
			//System.out.println("Client: " + agent.playerId + " Zooming out");
			// Zooming out
			try {
				Thread.sleep(2000);
			}
			catch (InterruptedException e1) 
			{

				e1.printStackTrace();
			}
			for (int k = 0; k < 15; k++) 
			{
				server.send(chrome_window, new ProxyMouseWheelMessage(-1));
			}
			try {
				Thread.sleep(2500);
			} catch (InterruptedException e1) {

				e1.printStackTrace();
			}
			int code = verifyLevel(level);
			if (code == PASS)
				return true;
			else
				if (code == ERROR_NOT_FULLY_ZOOM)
				{
					System.out.println("Client: " + agent.playerId + " Not Properly Zoomed out: Re-zoom");
					for (int k = 0; k < 15; k++) 
					{
						server.send(chrome_window, new ProxyMouseWheelMessage(-1));
					}
					try {
						Thread.sleep(2500);
					} catch (InterruptedException e1) {

						e1.printStackTrace();
					}
				}
				else
					if (code == ERROR_MISMATCHED_LEVEL)
					{
						loadLevel(level);
					}
			
		}
		return true;
	}
	private int verifyLevel(int requestedLevel)
	{
		byte[] imageBytes = server.send(chrome_window, new ProxyScreenshotMessage());
		BufferedImage image = null;
		try
		{
			image = ImageIO.read(new ByteArrayInputStream(imageBytes));
			Vision vision = new Vision(image);
			List<Rectangle> pigs = vision.findPigs();
			if (pigs.isEmpty())
				return ERROR_NOT_FULLY_ZOOM;
			// Get left most pig
			if (LEVEL_VERFICATION.size() == 0)
				return PASS;
			sortByX(pigs);
			Rectangle lmPig = pigs.get(0); //get the leftmost pig
			int actualLevel = matchLevelByLoop(lmPig.x, lmPig.y);
			if (actualLevel != requestedLevel)
			{	
				System.out.println("Client: " + agent.playerId + " MisMatchedLevel! Reqeust: " + requestedLevel 
						+"  Loaded: " + actualLevel);
				return ERROR_MISMATCHED_LEVEL;
			}
			
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		
		return PASS;
	}
/*	public int matchLevelByUtil(final Point pigCenter)
	{
		List<Point> verification = new ArrayList<Point>();
		verification.addAll(LEVEL_VERFICATION);
		Collections.sort(verification, new Comparator<Point>()
		{
			@Override
			public int compare(Point p1, Point p2) {
				Integer dis1 = (p1.x - pigCenter.x) * (p1.x - pigCenter.x) + (p1.y - pigCenter.y) * (p1.y - pigCenter.y);
				Integer dis2 = (p2.x - pigCenter.x) * (p2.x - pigCenter.x) + (p2.y - pigCenter.y) * (p2.y - pigCenter.y);
				return dis1.compareTo(dis2);
			}
			
		});
		return LEVEL_VERFICATION.indexOf(verification.get(0)) + 1;
		
	}
	*/
	
	private int matchLevelByLoop(int leftupX, int leftupY)
	{
		int counter = 1; // level start from 1.
		int min = Integer.MAX_VALUE;
		int minIndex = -1;
		for (Point p : LEVEL_VERFICATION)
		{
			int diff = (p.x - leftupX) * (p.x - leftupX) + (p.y - leftupY) * (p.y - leftupY);
			if (min > diff)
			{	
				min = diff;
				minIndex = counter;
			}
			counter ++;
		}
		return minIndex;
	}
	
	
	private void sortByX(List<Rectangle> pigs)
	{
		Collections.sort(pigs, new Comparator<Rectangle>(){

			@Override
			public int compare(Rectangle o1, Rectangle o2) {
				
				return ((Integer)(o1.x)).compareTo((Integer)(o2.x));
			}
			
			
		});
	}
	
/*	public static void main(String args[])
	{
		List<Point> veriPoints = new ArrayList<Point>();
		for (int i = 0; i < 21; i++)
		{
			veriPoints.add(new Point(i * 9 + 1, i * 22 - 3));
		}
		LoadingLevelSchema.setLevel_Verification(veriPoints); 
		LoadingLevelSchema schema = new LoadingLevelSchema();
		long time = System.currentTimeMillis();
		Point pigCenter = new Point(52, 21);
		int index = 0;
		for (int i = 100000; i > 0; i--)
		{
			schema.matchLevelByLoop(pigCenter.x , pigCenter.y);
		}
		index = schema.matchLevelByLoop(pigCenter.x, pigCenter.y);
		System.out.println(System.currentTimeMillis() - time + "  " + index);
		time = System.currentTimeMillis();
		for (int i = 100000; i > 0; i--)
		{
			schema.matchLevelByUtil(pigCenter);
		}
		index = schema.matchLevelByUtil(pigCenter);
		System.out.println(System.currentTimeMillis() - time + "  " + index);
		
	}*/
	
}
